@@ -1,5 +1,6 @@
 from django.conf import settings
 from django.contrib.auth.models import User
+from django.core.cache import cache
 from django.core.paginator import Paginator, EmptyPage
 from django.core.urlresolvers import reverse
 from django.http import HttpResponsePermanentRedirect, Http404, HttpResponseRedirect, HttpResponse
@@ -16,11 +17,11 @@ def display_blog_page(request, tag=None, username=None, year=None, month=None, p
     Yes, it's dirty to have so many URLs go to one view, but I'd rather do that
     than duplicate a bunch of code.  I'll probably revisit this in the future.
     """
-    context = {}
 
+    context = {}
     if tag:
         tag = get_object_or_404(Tag, name__iexact=tag)
-        articles = tag.article_set.active()
+        articles = tag.article_set.active().select_related()
         template = 'articles/display_tag.html'
         context['tag'] = tag
 
@@ -35,7 +36,7 @@ def display_blog_page(request, tag=None, username=None, year=None, month=None, p
         # listing articles in a given month and year
         year = int(year)
         month = int(month)
-        articles = Article.objects.active().filter(publish_date__year=year, publish_date__month=month)
+        articles = Article.objects.active().select_related().filter(publish_date__year=year, publish_date__month=month)
         template = 'articles/in_month.html'
         context['month'] = datetime(year, month, 1)
 
@@ -55,8 +56,9 @@ def display_blog_page(request, tag=None, username=None, year=None, month=None, p
     context.update({'paginator': paginator,
                     'page_obj': page})
     variables = RequestContext(request, context)
+    response = render_to_response(template, variables)
 
-    return render_to_response(template, variables)
+    return response
 
 def display_article(request, year, slug, template='articles/article_detail.html'):
     """Displays a single article."""
@@ -74,7 +76,9 @@ def display_article(request, year, slug, template='articles/article_detail.html'
         'article': article,
         'disqus_forum': getattr(settings, 'DISQUS_FORUM_SHORTNAME', None),
     })
-    return render_to_response(template, variables)
+    response = render_to_response(template, variables)
+
+    return response
 
 def redirect_to_article(request, year, month, day, slug):
     # this is a little snippet to handle URLs that are formatted the old way.
@@ -85,8 +89,18 @@ def ajax_tag_autocomplete(request):
     """Offers a list of existing tags that match the specified query"""
 
     if 'q' in request.GET:
-        tags = Tag.objects.filter(name__istartswith=request.GET['q'])[:10]
-        return HttpResponse(u'\n'.join(tag.name for tag in tags))
+        q = request.GET['q']
+        key = 'ajax_tag_auto_%s' % q
+        response = cache.get(key)
+
+        if response is not None:
+            return response
+
+        tags = list(Tag.objects.filter(name__istartswith=q)[:10])
+        response = HttpResponse(u'\n'.join(tag.name for tag in tags))
+        cache.set(key, response, 300)
+
+        return response
 
     return HttpResponse()
 
